import unittest

import reactivex
from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestWindow(unittest.TestCase):
    def test_window_when_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        window = [1]

        def create():
            def closing():
                curr = window[0]
                window[0] += 1
                return reactivex.timer(curr * 100)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_when(closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)
        assert results.messages == [
            on_next(250, "0 3"),
            on_next(260, "0 4"),
            on_next(310, "1 5"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(420, "1 8"),
            on_next(470, "1 9"),
            on_next(550, "2 10"),
            on_completed(590),
        ]
        assert xs.subscriptions == [subscribe(200, 590)]

    def test_window_when_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        window = [1]

        def create():
            def closing():
                curr = window[0]
                window[0] += 1
                return reactivex.timer(curr * 100)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_when(closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create, disposed=400)

        assert results.messages == [
            on_next(250, "0 3"),
            on_next(260, "0 4"),
            on_next(310, "1 5"),
            on_next(340, "1 6"),
        ]
        assert xs.subscriptions == [subscribe(200, 400)]

    def test_window_when_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_error(590, ex),
        )
        window = [1]

        def create():
            def closing():
                curr = window[0]
                window[0] += 1
                return reactivex.timer(curr * 100)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_when(closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(250, "0 3"),
            on_next(260, "0 4"),
            on_next(310, "1 5"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(420, "1 8"),
            on_next(470, "1 9"),
            on_next(550, "2 10"),
            on_error(590, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 590)]

    def test_window_when_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )

        def create():
            def closing():
                raise Exception(ex)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_when(closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [on_error(200, ex)]
        assert xs.subscriptions == [subscribe(200, 200)]

    def test_window_when_window_close_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )

        def create():
            def closing():
                return reactivex.throw(ex)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_when(closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [on_error(200, ex)]
        assert xs.subscriptions == [subscribe(200, 200)]

    def test_window_toggle_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        ys = scheduler.create_hot_observable(
            on_next(255, 50),
            on_next(330, 100),
            on_next(350, 50),
            on_next(400, 90),
            on_completed(900),
        )

        def create():
            def closing(x):
                return reactivex.timer(x)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_toggle(ys, closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)
        assert results.messages == [
            on_next(260, "0 4"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(410, "3 7"),
            on_next(420, "1 8"),
            on_next(420, "3 8"),
            on_next(470, "3 9"),
            on_completed(900),
        ]
        assert xs.subscriptions == [subscribe(200, 590)]
        assert ys.subscriptions == [subscribe(200, 900)]

    def test_window_toggle_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        ys = scheduler.create_hot_observable(
            on_next(255, 50),
            on_next(330, 100),
            on_next(350, 50),
            on_next(400, 90),
            on_completed(900),
        )

        def create():
            def closing(x):
                raise Exception(ex)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_toggle(ys, closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [on_error(255, ex)]
        assert xs.subscriptions == [subscribe(200, 255)]
        assert ys.subscriptions == [subscribe(200, 255)]

    def test_window_toggle_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        ys = scheduler.create_hot_observable(
            on_next(255, 50),
            on_next(330, 100),
            on_next(350, 50),
            on_next(400, 90),
            on_completed(900),
        )

        def create():
            def closing(x):
                return reactivex.timer(x)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_toggle(ys, closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create, disposed=415)
        assert results.messages == [
            on_next(260, "0 4"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(410, "3 7"),
        ]
        assert xs.subscriptions == [subscribe(200, 415)]
        assert ys.subscriptions == [subscribe(200, 415)]

    def test_window_toggle_data_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_error(415, ex),
        )
        ys = scheduler.create_hot_observable(
            on_next(255, 50),
            on_next(330, 100),
            on_next(350, 50),
            on_next(400, 90),
            on_completed(900),
        )

        def create():
            def closing(x):
                return reactivex.timer(x)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_toggle(ys, closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(260, "0 4"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(410, "3 7"),
            on_error(415, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 415)]
        assert ys.subscriptions == [subscribe(200, 415)]

    def test_window_toggle_window_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )
        ys = scheduler.create_hot_observable(
            on_next(255, 50),
            on_next(330, 100),
            on_next(350, 50),
            on_next(400, 90),
            on_error(415, ex),
        )

        def create():
            def closing(x):
                return reactivex.timer(x)

            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window_toggle(ys, closing),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(260, "0 4"),
            on_next(340, "1 6"),
            on_next(410, "1 7"),
            on_next(410, "3 7"),
            on_error(415, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 415)]
        assert ys.subscriptions == [subscribe(200, 415)]

    def test_window_simple(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )

        ys = scheduler.create_hot_observable(
            on_next(255, True),
            on_next(330, True),
            on_next(350, True),
            on_next(400, True),
            on_next(500, True),
            on_completed(900),
        )

        def create():
            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window(ys),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        res = scheduler.start(create=create)

        assert res.messages == [
            on_next(250, "0 3"),
            on_next(260, "1 4"),
            on_next(310, "1 5"),
            on_next(340, "2 6"),
            on_next(410, "4 7"),
            on_next(420, "4 8"),
            on_next(470, "4 9"),
            on_next(550, "5 10"),
            on_completed(590),
        ]

        assert xs.subscriptions == [subscribe(200, 590)]

        assert ys.subscriptions == [subscribe(200, 590)]

    def test_window_close_boundaries(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )

        ys = scheduler.create_hot_observable(
            on_next(255, True),
            on_next(330, True),
            on_next(350, True),
            on_completed(400),
        )

        def create():
            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window(ys),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        res = scheduler.start(create=create)

        assert res.messages == [
            on_next(250, "0 3"),
            on_next(260, "1 4"),
            on_next(310, "1 5"),
            on_next(340, "2 6"),
            on_completed(400),
        ]

        assert xs.subscriptions == [subscribe(200, 400)]

        assert ys.subscriptions == [subscribe(200, 400)]

    def test_window_throwSource(self):
        ex = "ex"
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(380, 7),
            on_error(400, ex),
        )

        ys = scheduler.create_hot_observable(
            on_next(255, True),
            on_next(330, True),
            on_next(350, True),
            on_completed(500),
        )

        def create():
            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window(ys),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        res = scheduler.start(create=create)

        assert res.messages == [
            on_next(250, "0 3"),
            on_next(260, "1 4"),
            on_next(310, "1 5"),
            on_next(340, "2 6"),
            on_next(380, "3 7"),
            on_error(400, ex),
        ]

        assert xs.subscriptions == [subscribe(200, 400)]

        assert ys.subscriptions == [subscribe(200, 400)]

    def test_window_throw_boundaries(self):
        ex = "ex"
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(90, 1),
            on_next(180, 2),
            on_next(250, 3),
            on_next(260, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(410, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_next(550, 10),
            on_completed(590),
        )

        ys = scheduler.create_hot_observable(
            on_next(255, True),
            on_next(330, True),
            on_next(350, True),
            on_error(400, ex),
        )

        def create():
            def mapper(w, i):
                return w.pipe(ops.map(lambda x: str(i) + " " + str(x)))

            return xs.pipe(
                ops.window(ys),
                ops.map_indexed(mapper),
                ops.merge_all(),
            )

        res = scheduler.start(create=create)

        assert res.messages == [
            on_next(250, "0 3"),
            on_next(260, "1 4"),
            on_next(310, "1 5"),
            on_next(340, "2 6"),
            on_error(400, ex),
        ]

        assert xs.subscriptions == [subscribe(200, 400)]

        assert ys.subscriptions == [subscribe(200, 400)]


if __name__ == "__main__":
    unittest.main()



import sys

import pytest

from reactivex.internal.exceptions import DisposedException
from reactivex.subject import ReplaySubject
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


def test_infinite():
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_next(630, 8),
        on_next(710, 9),
        on_next(870, 10),
        on_next(940, 11),
        on_next(1020, 12),
    )

    subject = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(3, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == [
        on_next(300, 3),
        on_next(300, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
    ]

    assert results2.messages == [
        on_next(400, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_next(630, 8),
    ]

    assert results3.messages == [on_next(900, 10), on_next(940, 11)]


def test_infinite2():
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(280, -1),
        on_next(290, -2),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_next(630, 8),
        on_next(710, 9),
        on_next(870, 10),
        on_next(940, 11),
        on_next(1020, 12),
    )

    subject = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(3, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == [
        on_next(300, 4),
        on_next(300, -1),
        on_next(300, -2),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
    ]

    assert results2.messages == [
        on_next(400, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_next(630, 8),
    ]

    assert results3.messages == [on_next(900, 10), on_next(940, 11)]


def test_finite():
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(630),
        on_next(640, 9),
        on_completed(650),
        on_error(660, "ex"),
    )

    subject = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(3, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action3(scheduler, state=None):
        subscription[0] = xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action3)

    def action4(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action4)

    def action5(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action5)

    def action6(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action6)

    def action7(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(900, action7)

    def action8(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action8)

    def action9(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action9)

    def action10(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action10)

    def action11(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action11)

    scheduler.start()

    assert results1.messages == [
        on_next(300, 3),
        on_next(300, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
    ]

    assert results2.messages == [
        on_next(400, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(630),
    ]

    assert results3.messages == [on_completed(900)]


def test_error():
    scheduler = TestScheduler()

    ex = RxException("ex")

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_error(630, ex),
        on_next(640, 9),
        on_completed(650),
        on_error(660, RxException("ex")),
    )

    subject = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(3, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == [
        on_next(300, 3),
        on_next(300, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
    ]

    assert results2.messages == [
        on_next(400, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_error(630, ex),
    ]

    assert results3.messages == [on_error(900, ex)]


def test_canceled():
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_completed(630),
        on_next(640, 9),
        on_completed(650),
        on_error(660, RxException()),
    )

    subject = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(3, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == []

    assert results2.messages == [on_completed(630)]

    assert results3.messages == [on_completed(900)]


def test_subject_disposed():
    scheduler = TestScheduler()

    subject = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(scheduler=scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription1[0] = subject[0].subscribe(results1)

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription2[0] = subject[0].subscribe(results2)

    scheduler.schedule_absolute(300, action3)

    def action4(scheduler, state=None):
        subscription3[0] = subject[0].subscribe(results3)

    scheduler.schedule_absolute(400, action4)

    def action5(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(500, action5)

    def action6(scheduler, state=None):
        subject[0].dispose()

    scheduler.schedule_absolute(600, action6)

    def action7(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action7)

    def action8(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(800, action8)

    def action9(scheduler, state=None):
        subject[0].on_next(1)

    scheduler.schedule_absolute(150, action9)

    def action10(scheduler, state=None):
        subject[0].on_next(2)

    scheduler.schedule_absolute(250, action10)

    def action11(scheduler, state=None):
        subject[0].on_next(3)

    scheduler.schedule_absolute(350, action11)

    def action12(scheduler, state=None):
        subject[0].on_next(4)

    scheduler.schedule_absolute(450, action12)

    def action13(scheduler, state=None):
        subject[0].on_next(5)

    scheduler.schedule_absolute(550, action13)

    def action14(scheduler, state=None):
        with pytest.raises(DisposedException):
            subject[0].on_next(6)

    scheduler.schedule_absolute(650, action14)

    def action15(scheduler, state=None):
        with pytest.raises(DisposedException):
            subject[0].on_completed()

    scheduler.schedule_absolute(750, action15)

    def action16(scheduler, state=None):
        with pytest.raises(DisposedException):
            subject[0].on_error(Exception())

    scheduler.schedule_absolute(850, action16)

    def action17(scheduler, state=None):
        with pytest.raises(DisposedException):
            subject[0].subscribe(None)

    scheduler.schedule_absolute(950, action17)

    scheduler.start()

    assert results1.messages == [
        on_next(200, 1),
        on_next(250, 2),
        on_next(350, 3),
        on_next(450, 4),
    ]

    assert results2.messages == [
        on_next(300, 1),
        on_next(300, 2),
        on_next(350, 3),
        on_next(450, 4),
        on_next(550, 5),
    ]

    assert results3.messages == [
        on_next(400, 1),
        on_next(400, 2),
        on_next(400, 3),
        on_next(450, 4),
        on_next(550, 5),
    ]


def test_replay_subject_dies_out():
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(580),
    )

    subject = [None]

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()
    results4 = scheduler.create_observer()

    def action1(scheduler, state=None):
        subject[0] = ReplaySubject(sys.maxsize, 100, scheduler)

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        xs.subscribe(subject[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subject[0].subscribe(results1)

    scheduler.schedule_absolute(300, action3)

    def action4(scheduler, state=None):
        subject[0].subscribe(results2)

    scheduler.schedule_absolute(400, action4)

    def action5(scheduler, state=None):
        subject[0].subscribe(results3)

    scheduler.schedule_absolute(600, action5)

    def action6(scheduler, state=None):
        subject[0].subscribe(results4)

    scheduler.schedule_absolute(900, action6)

    scheduler.start()

    assert results1.messages == [
        on_next(300, 3),
        on_next(300, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(580),
    ]

    assert results2.messages == [
        on_next(400, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(580),
    ]

    assert results3.messages == [on_next(600, 7), on_completed(600)]

    assert results4.messages == [on_completed(900)]



import unittest

import reactivex
from reactivex import operators as _
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestMaterialize(unittest.TestCase):
    def test_materialize_never(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.never().pipe(_.materialize())

        results = scheduler.start(create)
        assert results.messages == []

    def test_materialize_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(_.materialize())

        results = scheduler.start(create).messages
        assert len(results) == 2
        assert (
            results[0].value.kind == "N"
            and results[0].value.value.kind == "C"
            and results[0].time == 250
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_materialize_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_completed(250)
        )

        def create():
            return xs.pipe(_.materialize())

        results = scheduler.start(create).messages
        assert len(results) == 3
        assert (
            results[0].value.kind == "N"
            and results[0].value.value.kind == "N"
            and results[0].value.value.value == 2
            and results[0].time == 210
        )
        assert (
            results[1].value.kind == "N"
            and results[1].value.value.kind == "C"
            and results[1].time == 250
        )
        assert results[2].value.kind == "C" and results[1].time == 250

    def test_materialize_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))

        def create():
            return xs.pipe(_.materialize())

        results = scheduler.start(create).messages
        assert len(results) == 2
        assert (
            results[0].value.kind == "N"
            and results[0].value.value.kind == "E"
            and str(results[0].value.value.exception) == ex
        )
        assert results[1].value.kind == "C"

    def test_materialize_dematerialize_never(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.never().pipe(_.materialize(), _.dematerialize())

        results = scheduler.start(create)
        assert results.messages == []

    def test_materialize_dematerialize_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(_.materialize(), _.dematerialize())

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert results[0].value.kind == "C" and results[0].time == 250

    def test_materialize_dematerialize_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_completed(250)
        )

        def create():
            return xs.pipe(_.materialize(), _.dematerialize())

        results = scheduler.start(create).messages
        assert len(results) == 2
        assert (
            results[0].value.kind == "N"
            and results[0].value.value == 2
            and results[0].time == 210
        )
        assert results[1].value.kind == "C"

    def test_materialize_dematerialize_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))

        def create():
            return xs.pipe(_.materialize(), _.dematerialize())

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert (
            results[0].value.kind == "E"
            and str(results[0].value.exception) == ex
            and results[0].time == 250
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestMaxBy(unittest.TestCase):
    def test_maxby_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, {"key": 1, "value": "z"}), on_completed(250)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            def mapper(x):
                return x["key"]

            return xs.pipe(ops.max_by(mapper))

        res = scheduler.start(create=create).messages
        self.assertEqual(2, len(res))
        self.assertEqual(0, len(res[0].value.value))
        assert res[1].value.kind == "C" and res[1].time == 250

    def test_maxby_return(self):
        scheduler = TestScheduler()
        msgs = [
            on_next(150, {"key": 1, "value": "z"}),
            on_next(210, {"key": 2, "value": "a"}),
            on_completed(250),
        ]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            def mapper(x):
                return x["key"]

            return xs.pipe(ops.max_by(mapper))

        res = scheduler.start(create=create).messages
        self.assertEqual(2, len(res))
        assert res[0].value.kind == "N"
        self.assertEqual(1, len(res[0].value.value))
        self.assertEqual(2, res[0].value.value[0]["key"])
        self.assertEqual("a", res[0].value.value[0]["value"])
        assert res[1].value.kind == "C" and res[1].time == 250

    def test_maxby_some(self):
        scheduler = TestScheduler()
        msgs = [
            on_next(150, {"key": 1, "value": "z"}),
            on_next(210, {"key": 3, "value": "b"}),
            on_next(220, {"key": 4, "value": "c"}),
            on_next(230, {"key": 2, "value": "a"}),
            on_completed(250),
        ]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            def mapper(x):
                return x["key"]

            return xs.pipe(ops.max_by(mapper))

        res = scheduler.start(create=create).messages
        self.assertEqual(2, len(res))
        assert res[0].value.kind == "N"
        self.assertEqual(1, len(res[0].value.value[0]["value"]))
        self.assertEqual(4, res[0].value.value[0]["key"])
        self.assertEqual("c", res[0].value.value[0]["value"])
        assert res[1].value.kind == "C" and res[1].time == 250

    def test_maxby_multiple(self):
        scheduler = TestScheduler()
        msgs = [
            on_next(150, {"key": 1, "value": "z"}),
            on_next(210, {"key": 3, "value": "b"}),
            on_next(215, {"key": 2, "value": "d"}),
            on_next(220, {"key": 3, "value": "c"}),
            on_next(225, {"key": 2, "value": "y"}),
            on_next(230, {"key": 4, "value": "a"}),
            on_next(235, {"key": 4, "value": "r"}),
            on_completed(250),
        ]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.max_by(lambda x: x["key"]))

        res = scheduler.start(create=create).messages

        self.assertEqual(2, len(res))
        assert res[0].value.kind == "N"
        self.assertEqual(2, len(res[0].value.value))
        self.assertEqual(4, res[0].value.value[0]["key"])
        self.assertEqual("a", res[0].value.value[0]["value"])
        self.assertEqual(4, res[0].value.value[1]["key"])
        self.assertEqual("r", res[0].value.value[1]["value"])
        assert res[1].value.kind == "C" and res[1].time == 250

    def test_maxby_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs = [on_next(150, {"key": 1, "value": "z"}), on_error(210, ex)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.max_by(lambda x: x["key"]))

        res = scheduler.start(create=create).messages

        assert res == [on_error(210, ex)]

    def test_maxby_never(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, {"key": 1, "value": "z"})]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.max_by(lambda x: x["key"]))

        res = scheduler.start(create=create).messages
        assert res == []

    # def test_MaxBy_Comparer_Empty():
    #     var msgs, res, reverseComparer, scheduler, xs
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         }),
    #         on_completed(250)
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             return x.key
    #         }, reverseComparer)
    #     }).messages
    #     self.assertEqual(2, res.length)
    #     self.assertEqual(0, res[0].value.value.length)
    #     assert(res[1].value.kind == 'C' and res[1].time == 250)

    # def test_MaxBy_Comparer_Return():
    #     var msgs, res, reverseComparer, scheduler, xs
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         }), on_next(210, {
    #             key: 2,
    #             value: 'a'
    #         }), on_completed(250)
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             return x.key
    #         }, reverseComparer)
    #     }).messages
    #     self.assertEqual(2, res.length)
    #     assert(res[0].value.kind == 'N')
    #     self.assertEqual(1, res[0].value.value.length)
    #     self.assertEqual(2, res[0].value.value[0].key)
    #     self.assertEqual('a', res[0].value.value[0].value)
    #     assert(res[1].value.kind == 'C' and res[1].time == 250)

    # def test_MaxBy_Comparer_Some():
    #     var msgs, res, reverseComparer, scheduler, xs
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         }), on_next(210, {
    #             key: 3,
    #             value: 'b'
    #         }), on_next(220, {
    #             key: 4,
    #             value: 'c'
    #         }), on_next(230, {
    #             key: 2,
    #             value: 'a'
    #         }), on_completed(250)
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             return x.key
    #         }, reverseComparer)
    #     }).messages
    #     self.assertEqual(2, res.length)
    #     assert(res[0].value.kind == 'N')
    #     self.assertEqual(1, res[0].value.value.length)
    #     equal(2, res[0].value.value[0].key)
    #     self.assertEqual('a', res[0].value.value[0].value)
    #     assert(res[1].value.kind == 'C' and res[1].time == 250)

    # def test_MaxBy_Comparer_Throw():
    #     var ex, msgs, res, reverseComparer, scheduler, xs
    #     ex = 'ex'
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         }), on_error(210, ex)
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             return x.key
    #         }, reverseComparer)
    #     }).messages
    #     assert res == [on_error(210, ex)]

    # def test_MaxBy_Comparer_Never():
    #     var msgs, res, reverseComparer, scheduler, xs
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         })
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             return x.key
    #         }, reverseComparer)
    #     }).messages
    #     assert res == []

    # def test_MaxBy_SelectorThrows():
    #     var ex, msgs, res, reverseComparer, scheduler, xs
    #     ex = 'ex'
    #     scheduler = TestScheduler()
    #     msgs = [
    #         on_next(150, {
    #             key: 1,
    #             value: 'z'
    #         }), on_next(210, {
    #             key: 3,
    #             value: 'b'
    #         }), on_next(220, {
    #             key: 2,
    #             value: 'c'
    #         }), on_next(230, {
    #             key: 4,
    #             value: 'a'
    #         }), on_completed(250)
    #     ]
    #     reverseComparer = function (a, b) {
    #         if (a > b) {
    #             return -1
    #         }
    #         if (a < b) {
    #             return 1
    #         }
    #         return 0
    #     }
    #     xs = scheduler.create_hot_observable(msgs)
    #     res = scheduler.start(create=create)
    #         return xs.max_by(function (x) {
    #             throw ex
    #         }, reverseComparer)
    #     }).messages
    #     assert res == [on_error(210, ex)]

    def test_maxby_comparerthrows(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs = [
            on_next(150, {"key": 1, "value": "z"}),
            on_next(210, {"key": 3, "value": "b"}),
            on_next(220, {"key": 2, "value": "c"}),
            on_next(230, {"key": 4, "value": "a"}),
            on_completed(250),
        ]

        def reverse_comparer(a, b):
            raise Exception(ex)

        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.max_by(lambda x: x["key"], reverse_comparer))

        res = scheduler.start(create=create).messages
        assert res == [on_error(220, ex)]



import unittest

from reactivex import operators as _
from reactivex.observable.observable import Observable
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestLastOrDefault(unittest.TestCase):
    def test_last_or_default_async_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create() -> Observable[int]:
            return xs.pipe(_.last_or_default(default_value=0))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 0), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_or_default_async(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_completed(250)
        )

        def create() -> Observable[int]:
            return xs.pipe(_.last_or_default(0))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 2), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_or_default_async_many(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_completed(250),
        )

        def create():
            return xs.pipe(_.last_or_default(0))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 3), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_or_default_async_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))

        def create():
            return xs.pipe(_.last_or_default(0))

        res = scheduler.start(create=create)

        assert res.messages == [on_error(210, ex)]
        assert xs.subscriptions == [subscribe(200, 210)]

    def test_last_or_default_async_predicate(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create() -> Observable[int]:
            def predicate(x: int) -> bool:
                return x % 2 == 1

            return xs.pipe(_.last_or_default(0, predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 5), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_or_default_async_Predicate_none(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            def predicate(x: int) -> bool:
                return x > 10

            return xs.pipe(_.last_or_default(0, predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(250, 0), on_completed(250)]
        assert xs.subscriptions == [subscribe(200, 250)]

    def test_last_or_default_async_Predicate_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))

        def create():
            def predicate(x: int) -> bool:
                return x > 10

            return xs.pipe(_.last_or_default(0, predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_error(210, ex)]
        assert xs.subscriptions == [subscribe(200, 210)]

    def test_last_or_default_async_predicate_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            def predicate(x: int) -> bool:
                if x < 4:
                    return x % 2 == 1
                else:
                    raise Exception(ex)

            return xs.pipe(_.last_or_default(0, predicate))

        res = scheduler.start(create=create)

        assert res.messages == [on_error(230, ex)]
        assert xs.subscriptions == [subscribe(200, 230)]



import unittest

import reactivex
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestCase(unittest.TestCase):
    def test_case_one(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        zs = scheduler.create_hot_observable(
            on_next(230, 21), on_next(240, 22), on_next(290, 23), on_completed(320)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 1, map, zs)

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 1),
            on_next(240, 2),
            on_next(270, 3),
            on_completed(300),
        ]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == []
        assert zs.subscriptions == []

    def test_case_two(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        zs = scheduler.create_hot_observable(
            on_next(230, 21), on_next(240, 22), on_next(290, 23), on_completed(320)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 2, map, zs)

        results = scheduler.start(create)

        assert results.messages == [
            on_next(220, 11),
            on_next(250, 12),
            on_next(280, 13),
            on_completed(310),
        ]
        assert xs.subscriptions == []
        assert ys.subscriptions == [subscribe(200, 310)]
        assert zs.subscriptions == []

    def test_case_three(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        zs = scheduler.create_hot_observable(
            on_next(230, 21), on_next(240, 22), on_next(290, 23), on_completed(320)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 3, map, zs)

        results = scheduler.start(create)

        assert results.messages == [
            on_next(230, 21),
            on_next(240, 22),
            on_next(290, 23),
            on_completed(320),
        ]
        assert xs.subscriptions == []
        assert ys.subscriptions == []
        assert zs.subscriptions == [subscribe(200, 320)]

    def test_case_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        zs = scheduler.create_hot_observable(
            on_next(230, 21), on_next(240, 22), on_next(290, 23), on_completed(320)
        )
        map = {1: xs, 2: ys}

        def create():
            def mapper():
                raise Exception(ex)

            return reactivex.case(mapper, map, zs)

        results = scheduler.start(create)

        assert results.messages == [on_error(200, ex)]
        assert xs.subscriptions == []
        assert ys.subscriptions == []
        assert zs.subscriptions == []

    def test_case_with_default_one(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 1, map)

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(210, 1),
            on_next(240, 2),
            on_next(270, 3),
            on_completed(300),
        ]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == []

    def test_case_with_default_two(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 2, map)

        results = scheduler.start(create=create)

        assert results.messages == [
            on_next(220, 11),
            on_next(250, 12),
            on_next(280, 13),
            on_completed(310),
        ]
        assert xs.subscriptions == []
        assert ys.subscriptions == [subscribe(200, 310)]

    def test_case_with_default_three(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        map = {1: xs, 2: ys}

        def create():
            return reactivex.case(lambda: 3, map)

        results = scheduler.start(create=create)

        assert results.messages == [on_completed(200)]
        assert xs.subscriptions == []
        assert ys.subscriptions == []

    def test_case_with_default_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(240, 2), on_next(270, 3), on_completed(300)
        )
        ys = scheduler.create_hot_observable(
            on_next(220, 11), on_next(250, 12), on_next(280, 13), on_completed(310)
        )
        map = {1: xs, 2: ys}

        def create():
            def mapper():
                raise Exception(ex)

            return reactivex.case(mapper, map)

        results = scheduler.start(create)

        assert results.messages == [on_error(200, ex)]
        assert xs.subscriptions == []
        assert ys.subscriptions == []



import asyncio
import unittest
from asyncio import Future

import reactivex


class TestFromFuture(unittest.TestCase):
    def test_future_success(self):
        loop = asyncio.get_event_loop()
        success = [False, True, False]

        async def go():
            future = Future()
            future.set_result(42)

            source = reactivex.from_future(future)

            def on_next(x):
                success[0] = x == 42

            def on_error(err):
                success[1] = False

            def on_completed():
                success[2] = True

            source.subscribe(on_next, on_error, on_completed)

        loop.run_until_complete(go())
        assert all(success)

    def test_future_failure(self):
        loop = asyncio.get_event_loop()
        success = [True, False, True]

        async def go():
            error = Exception("woops")

            future = Future()
            future.set_exception(error)

            source = reactivex.from_future(future)

            def on_next(x):
                success[0] = False

            def on_error(err):
                success[1] = str(err) == str(error)

            def on_completed():
                success[2] = False

            source.subscribe(on_next, on_error, on_completed)

        loop.run_until_complete(go())
        assert all(success)

    def test_future_cancel(self):
        loop = asyncio.get_event_loop()
        success = [True, False, True]

        async def go():
            future = Future()
            source = reactivex.from_future(future)

            def on_next(x):
                success[0] = False

            def on_error(err):
                success[1] = type(err) == asyncio.CancelledError

            def on_completed():
                success[2] = False

            source.subscribe(on_next, on_error, on_completed)
            future.cancel()

        loop.run_until_complete(go())
        assert all(success)

    def test_future_dispose(self):
        loop = asyncio.get_event_loop()
        success = [True, True, True]

        async def go():
            future = Future()
            future.set_result(42)

            source = reactivex.from_future(future)

            def on_next(x):
                success[0] = False

            def on_error(err):
                success[1] = False

            def on_completed():
                success[2] = False

            subscription = source.subscribe(on_next, on_error, on_completed)
            subscription.dispose()

        loop.run_until_complete(go())
        assert all(success)



import unittest

from reactivex import throw
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestThrow(unittest.TestCase):
    def test_throw_exception_basic(self):
        scheduler = TestScheduler()
        ex = "ex"

        def factory():
            return throw(ex)

        results = scheduler.start(factory)
        assert results.messages == [on_error(200, ex)]

    def test_throw_disposed(self):
        scheduler = TestScheduler()

        def factory():
            return throw("ex")

        results = scheduler.start(factory, disposed=200)
        assert results.messages == []

    def test_throw_observer_throws(self):
        scheduler = TestScheduler()
        xs = throw("ex")
        xs.subscribe(
            lambda x: None, lambda ex: _raise("ex"), lambda: None, scheduler=scheduler
        )

        self.assertRaises(RxException, scheduler.start)



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestDelayWithMapper(unittest.TestCase):
    # Delay with mapper
    def test_delay_duration_simple1(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 10),
            on_next(220, 30),
            on_next(230, 50),
            on_next(240, 35),
            on_next(250, 20),
            on_completed(260),
        )

        def create():
            def mapper(x):
                return scheduler.create_cold_observable(on_next(x, "!"))

            return xs.pipe(ops.delay_with_mapper(mapper))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210 + 10, 10),
            on_next(220 + 30, 30),
            on_next(250 + 20, 20),
            on_next(240 + 35, 35),
            on_next(230 + 50, 50),
            on_completed(280),
        ]
        assert xs.subscriptions == [subscribe(200, 260)]

    def test_delay_duration_simple2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_next(250, 6),
            on_completed(300),
        )
        ys = scheduler.create_cold_observable(on_next(10, "!"))

        def create():
            return xs.pipe(ops.delay_with_mapper(lambda _: ys))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210 + 10, 2),
            on_next(220 + 10, 3),
            on_next(230 + 10, 4),
            on_next(240 + 10, 5),
            on_next(250 + 10, 6),
            on_completed(300),
        ]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [
            subscribe(210, 220),
            subscribe(220, 230),
            subscribe(230, 240),
            subscribe(240, 250),
            subscribe(250, 260),
        ]

    def test_delay_duration_simple3(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_next(250, 6),
            on_completed(300),
        )
        ys = scheduler.create_cold_observable(on_next(100, "!"))

        def create():
            return xs.pipe(ops.delay_with_mapper(lambda _: ys))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210 + 100, 2),
            on_next(220 + 100, 3),
            on_next(230 + 100, 4),
            on_next(240 + 100, 5),
            on_next(250 + 100, 6),
            on_completed(350),
        ]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [
            subscribe(210, 310),
            subscribe(220, 320),
            subscribe(230, 330),
            subscribe(240, 340),
            subscribe(250, 350),
        ]

    def test_delay_duration_simple4_inner_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_next(250, 6),
            on_completed(300),
        )
        ys = scheduler.create_cold_observable(on_completed(100))

        def create():
            return xs.pipe(ops.delay_with_mapper(lambda _: ys))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210 + 100, 2),
            on_next(220 + 100, 3),
            on_next(230 + 100, 4),
            on_next(240 + 100, 5),
            on_next(250 + 100, 6),
            on_completed(350),
        ]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [
            subscribe(210, 310),
            subscribe(220, 320),
            subscribe(230, 330),
            subscribe(240, 340),
            subscribe(250, 350),
        ]

    def test_delay_duration_dispose1(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_next(250, 6),
            on_completed(300),
        )
        ys = scheduler.create_cold_observable(on_next(200, "!"))

        def create():
            return xs.pipe(ops.delay_with_mapper(lambda _: ys))

        results = scheduler.start(create, disposed=425)
        assert results.messages == [on_next(210 + 200, 2), on_next(220 + 200, 3)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [
            subscribe(210, 410),
            subscribe(220, 420),
            subscribe(230, 425),
            subscribe(240, 425),
            subscribe(250, 425),
        ]

    def test_delay_duration_dispose2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_next(400, 3), on_completed(500)
        )
        ys = scheduler.create_cold_observable(on_next(50, "!"))

        def create():
            return xs.pipe(ops.delay_with_mapper(lambda _: ys))

        results = scheduler.start(create, disposed=300)
        assert results.messages == [on_next(210 + 50, 2)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [subscribe(210, 260)]



import unittest

from reactivex.observable.observable import Observable
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSlice(unittest.TestCase):
    def test_slice_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_completed(250)]
        xs = scheduler.create_hot_observable(msgs)

        def create() -> Observable[int]:
            return xs[1:42]

        res = scheduler.start(create=create).messages
        assert res == [on_completed(250)]

    def test_slice_same(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[0:10]

        results = scheduler.start(create)
        assert results.messages == [
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(415),
        ]
        assert xs.subscriptions == [subscribe(200, 415)]

    def test_slice_same_noop(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[:]

        results = scheduler.start(create)
        assert results.messages == [
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        ]
        assert xs.subscriptions == [subscribe(200, 690)]

    def test_slice_skip_first(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[2:]

        results = scheduler.start(create)
        assert results.messages == [
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        ]
        assert xs.subscriptions == [subscribe(200, 690)]

    def test_slice_skip_last(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[:-2]

        results = scheduler.start(create)
        assert results.messages == [
            on_next(270, 0),
            on_next(280, 1),
            on_next(300, 2),
            on_next(310, 3),
            on_next(340, 4),
            on_next(370, 5),
            on_next(410, 6),
            on_next(415, 7),
            on_completed(690),
        ]
        assert xs.subscriptions == [subscribe(200, 690)]

    def test_slice_take_last(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[-2:]

        results = scheduler.start(create)
        assert results.messages == [on_next(690, 8), on_next(690, 9), on_completed(690)]
        assert xs.subscriptions == [subscribe(200, 690)]

    def test_slice_take_first(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[:2]

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 0), on_next(230, 1), on_completed(230)]
        assert xs.subscriptions == [subscribe(200, 230)]

    def test_slice_take_last_skip_all(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[-2:0]

        results = scheduler.start(create)
        assert results.messages == [on_completed(200)]

    def test_slice_step_2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(70, -2),
            on_next(150, -1),
            on_next(210, 0),
            on_next(230, 1),
            on_next(270, 2),
            on_next(280, 3),
            on_next(300, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(370, 7),
            on_next(410, 8),
            on_next(415, 9),
            on_completed(690),
        )

        def create():
            return xs[0:10:2]

        results = scheduler.start(create)
        assert results.messages == [
            on_next(210, 0),
            on_next(270, 2),
            on_next(300, 4),
            on_next(340, 6),
            on_next(410, 8),
            on_completed(415),
        ]
        assert xs.subscriptions == [subscribe(200, 415)]



import unittest
from typing import Any, List, Optional

from reactivex import abc
from reactivex import operators as ops
from reactivex.observable.connectableobservable import ConnectableObservable
from reactivex.subject import Subject
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestMulticast(unittest.TestCase):
    def test_multicast_hot_1(self):
        scheduler = TestScheduler()

        s: Subject[int] = Subject()

        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )

        obv = scheduler.create_observer()
        d1: List[Optional[abc.DisposableBase]] = [None]
        d2: List[Optional[abc.DisposableBase]] = [None]
        c: List[Optional[ConnectableObservable[int]]] = [None]

        def action(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action)

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            assert c[0]
            d1[0] = c[0].subscribe(obv, scheduler=scheduler)

        scheduler.schedule_absolute(100, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            assert c[0]
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(200, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            assert d1[0]
            d1[0].dispose()

        scheduler.schedule_absolute(300, action2)

        scheduler.start()

        assert obv.messages == [on_next(210, 3), on_next(240, 4), on_next(270, 5)]
        assert xs.subscriptions == [subscribe(200, 390)]

    def test_multicast_hot_2(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o, scheduler)

        scheduler.schedule_absolute(200, action2)

        def action3(scheduler: abc.SchedulerBase, state: Any = None):
            return d1[0].dispose()

        scheduler.schedule_absolute(300, action3)

        scheduler.start()
        assert o.messages == [on_next(210, 3), on_next(240, 4), on_next(270, 5)]
        assert xs.subscriptions == [subscribe(100, 390)]

    def test_multicast_hot_21(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o)

        scheduler.schedule_absolute(200, action2)

        def action3(scheduler: abc.SchedulerBase, state: Any = None):
            return d1[0].dispose()

        scheduler.schedule_absolute(300, action3)

        scheduler.start()
        assert o.messages == [on_next(210, 3), on_next(240, 4), on_next(270, 5)]
        assert xs.subscriptions == [subscribe(100, 390)]

    def test_multicast_hot_3(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o)

        scheduler.schedule_absolute(200, action2)

        def action3(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0].dispose()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(335, action4)

        scheduler.start()
        assert o.messages == [
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(340, 7),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(100, 300), subscribe(335, 390)]

    def test_multicast_hot_4(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_error(390, ex),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o, scheduler)

        scheduler.schedule_absolute(200, action2)

        def action3(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0].dispose()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(335, action4)

        scheduler.start()
        assert o.messages == [
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(340, 7),
            on_error(390, ex),
        ]
        assert xs.subscriptions == [subscribe(100, 300), subscribe(335, 390)]

    def test_multicast_hot_5(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_error(390, ex),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o, scheduler)

        scheduler.schedule_absolute(400, action2)

        scheduler.start()
        assert o.messages == [on_error(400, ex)]
        assert xs.subscriptions == [subscribe(100, 390)]

    def test_multicast_hot_6(self):
        c = [None]
        d1 = [None]
        d2 = [None]
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )
        s = Subject()
        o = scheduler.create_observer()

        def action0(scheduler: abc.SchedulerBase, state: Any = None):
            c[0] = xs.pipe(ops.multicast(s))

        scheduler.schedule_absolute(50, action0)

        def action1(scheduler: abc.SchedulerBase, state: Any = None):
            d2[0] = c[0].connect(scheduler)

        scheduler.schedule_absolute(100, action1)

        def action2(scheduler: abc.SchedulerBase, state: Any = None):
            d1[0] = c[0].subscribe(o, scheduler)

        scheduler.schedule_absolute(400, action2)

        scheduler.start()
        assert o.messages == [on_completed(400)]
        assert xs.subscriptions == [subscribe(100, 390)]

    def test_multicast_cold_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )

        def create():
            def subject_factory(scheduler):
                return Subject()

            def mapper(ys):
                return ys

            return xs.pipe(
                ops.multicast(subject_factory=subject_factory, mapper=mapper)
            )

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]

    def test_multicast_cold_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_error(390, ex),
        )

        def create():
            def subject_factory(scheduler):
                return Subject()

            def mapper(ys):
                return ys

            return xs.pipe(
                ops.multicast(subject_factory=subject_factory, mapper=mapper)
            )

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_error(390, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]

    def test_multicast_cold_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
        )

        def create():
            def subject_factory(scheduler):
                return Subject()

            def mapper(ys):
                return ys

            return xs.pipe(
                ops.multicast(subject_factory=subject_factory, mapper=mapper)
            )

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
        ]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_multicast_cold_zip(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )

        def create():
            def subject_factory(scheduler):
                return Subject()

            def mapper(ys):
                return ys.pipe(
                    ops.zip(ys),
                    ops.map(sum),
                )

            return xs.pipe(
                ops.multicast(subject_factory=subject_factory, mapper=mapper)
            )

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 6),
            on_next(240, 8),
            on_next(270, 10),
            on_next(330, 12),
            on_next(340, 14),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestMin(unittest.TestCase):
    def test_min_int32_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(ops.min())

        res = scheduler.start(create=create).messages
        assert 1 == len(res)
        assert res[0].value.kind == "E" and res[0].value.exception
        assert res[0].time == 250

    def test_min_int32_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_completed(250)
        )
        res = scheduler.start(create=lambda: xs.pipe(ops.min())).messages
        assert res == [on_next(250, 2), on_completed(250)]

    def test_min_int32_some(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_completed(250),
        )
        res = scheduler.start(create=lambda: xs.pipe(ops.min())).messages
        assert res == [on_next(250, 2), on_completed(250)]

    def test_min_int32_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))
        res = scheduler.start(create=lambda: xs.pipe(ops.min())).messages
        assert res == [on_error(210, ex)]

    def test_min_int32_never(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1))
        res = scheduler.start(create=lambda: xs.pipe(ops.min())).messages
        assert res == []

    def test_min_of_t_comparer_empty(self):
        scheduler = TestScheduler()

        def comparer(a, b):
            if a > b:
                return -1

            if a == b:
                return 0

            return 1

        xs = scheduler.create_hot_observable(on_next(150, "a"), on_completed(250))

        def create():
            return xs.pipe(ops.min(comparer))

        res = scheduler.start(create=create).messages
        self.assertEqual(1, len(res))
        assert res[0].value.kind == "E" and res[0].value.exception is not None
        assert res[0].time == 250

    def test_min_of_t_comparer_empty_ii(self):
        scheduler = TestScheduler()

        def comparer(a, b):
            if a > b:
                return -1

            if a == b:
                return 0

            return 1

        xs = scheduler.create_hot_observable(
            on_next(150, "z"),
            on_next(210, "b"),
            on_next(220, "c"),
            on_next(230, "a"),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.min(comparer))

        res = scheduler.start(create=create).messages
        assert res == [on_next(250, "c"), on_completed(250)]

    def test_min_of_t_comparer_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()

        def comparer(a, b):
            if a > b:
                return -1

            if a == b:
                return 0

            return 1

        xs = scheduler.create_hot_observable(on_next(150, "z"), on_error(210, ex))

        def create():
            return xs.pipe(ops.min(comparer))

        res = scheduler.start(create=create).messages
        assert res == [on_error(210, ex)]

    def test_min_of_t_comparer_never(self):
        scheduler = TestScheduler()

        def comparer(a, b):
            if a > b:
                return -1

            if a == b:
                return 0

            return 1

        xs = scheduler.create_hot_observable(on_next(150, "z"))

        def create():
            return xs.pipe(ops.min(comparer))

        res = scheduler.start(create=create).messages
        assert res == []

    def test_min_of_t_comparer_throws(self):
        ex = "ex"
        scheduler = TestScheduler()

        def comparer(a, b):
            raise Exception(ex)

        xs = scheduler.create_hot_observable(
            on_next(150, "z"),
            on_next(210, "b"),
            on_next(220, "c"),
            on_next(230, "a"),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.min(comparer))

        res = scheduler.start(create=create).messages
        assert res == [on_error(220, ex)]



import unittest

import reactivex
from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas


def _raise(ex: str) -> None:
    raise RxException(ex)


class TestWithLatestFrom(unittest.TestCase):
    def test_with_latest_from_never_never(self):
        scheduler = TestScheduler()
        e1 = reactivex.never()
        e2 = reactivex.never()

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == []

    def test_with_latest_from_never_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_completed(210)]
        e1 = reactivex.never()
        e2 = scheduler.create_hot_observable(msgs)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == []

    def test_with_latest_from_empty_never(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_completed(210)]
        e1 = reactivex.never()
        e2 = scheduler.create_hot_observable(msgs)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(210)]

    def test_with_latest_from_empty_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_completed(210)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(210)]

    def test_with_latest_from_empty_return(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(210)]

    def test_with_latest_from_return_empty(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(210)]
        msgs2 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(220)]

    def test_with_latest_from_never_return(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        e1 = scheduler.create_hot_observable(msgs)
        e2 = reactivex.never()

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(220)]

    def test_with_latest_from_return_never(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(215, 2), on_completed(210)]
        e1 = scheduler.create_hot_observable(msgs)
        e2 = reactivex.never()

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == []

    def test_with_latest_from_return_return(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_next(220, 3), on_completed(240)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(230)]

    def test_with_latest_from_empty_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_error_empty(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_return_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_throw_return(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_throw_on_error(self):
        ex1 = "ex1"
        ex2 = "ex2"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_error(220, ex1)]
        msgs2 = [on_next(150, 1), on_error(230, ex2)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex1)]

    def test_with_latest_from_error_on_error(self):
        ex1 = "ex1"
        ex2 = "ex2"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_error(220, ex1)]
        msgs2 = [on_next(150, 1), on_error(230, ex2)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex1)]

    def test_with_latest_from_throw_error(self):
        ex1 = "ex1"
        ex2 = "ex2"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(210, 2), on_error(220, ex1)]
        msgs2 = [on_next(150, 1), on_error(230, ex2)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex1)]

    def test_with_latest_from_never_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_error(220, ex)]
        e1 = reactivex.never()
        e2 = scheduler.create_hot_observable(msgs)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_throw_never(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_error(220, ex)]
        e1 = reactivex.never()
        e2 = scheduler.create_hot_observable(msgs)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_some_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_throw_some(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_error(220, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(220, ex)]

    def test_with_latest_from_no_throw_after_complete_left(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        msgs2 = [on_next(150, 1), on_error(230, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(220)]

    def test_with_latest_from_throw_after_complete_right(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_completed(220)]
        msgs2 = [on_next(150, 1), on_error(230, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(230, ex)]

    def test_with_latest_from_interleaved_with_tail(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_next(225, 4), on_completed(230)]
        msgs2 = [
            on_next(150, 1),
            on_next(220, 3),
            on_next(230, 5),
            on_next(235, 6),
            on_next(240, 7),
            on_completed(250),
        ]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_next(225, 3 + 4), on_completed(230)]

    def test_with_latest_from_consecutive(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_next(225, 4), on_completed(230)]
        msgs2 = [on_next(150, 1), on_next(235, 6), on_next(240, 7), on_completed(250)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_completed(230)]

    def test_with_latest_from_consecutive_end_with_error_left(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_next(225, 4), on_error(230, ex)]
        msgs2 = [on_next(150, 1), on_next(235, 6), on_next(240, 7), on_completed(250)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(230, ex)]

    def test_with_latest_from_consecutive_end_with_error_right(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(215, 2), on_next(225, 4), on_completed(230)]
        msgs2 = [on_next(150, 1), on_next(235, 6), on_next(240, 7), on_error(245, ex)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e2.pipe(
                ops.with_latest_from(e1),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [
            on_next(235, 4 + 6),
            on_next(240, 4 + 7),
            on_error(245, ex),
        ]

    def test_with_latest_from_mapper_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [on_next(150, 1), on_next(225, 2), on_completed(230)]
        msgs2 = [on_next(150, 1), on_next(220, 3), on_completed(240)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(lambda xy: _raise(ex)),
            )

        results = scheduler.start(create)
        assert results.messages == [on_error(225, ex)]

    def test_with_latest_from_repeat_last_left_value(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(150, 1),
            on_next(215, 2),
            on_next(225, 4),
            on_next(230, 5),
            on_completed(235),
        ]
        msgs2 = [on_next(150, 1), on_next(220, 3), on_completed(250)]
        e1 = scheduler.create_hot_observable(msgs1)
        e2 = scheduler.create_hot_observable(msgs2)

        def create():
            return e1.pipe(
                ops.with_latest_from(e2),
                ops.map(sum),
            )

        results = scheduler.start(create)
        assert results.messages == [
            on_next(225, 3 + 4),
            on_next(230, 3 + 5),
            on_completed(235),
        ]


if __name__ == "__main__":
    unittest.main()



import threading
import unittest
from datetime import timedelta
from time import sleep

import pytest

from reactivex.internal.basic import default_now
from reactivex.scheduler import CurrentThreadScheduler


class TestCurrentThreadScheduler(unittest.TestCase):
    def test_currentthread_singleton(self):
        scheduler = [
            CurrentThreadScheduler(),
            CurrentThreadScheduler.singleton(),
            CurrentThreadScheduler.singleton(),
        ]
        assert scheduler[0] is not scheduler[1]
        assert scheduler[1] is scheduler[2]

        gate = [threading.Semaphore(0), threading.Semaphore(0)]
        scheduler = [None, None]

        def run(idx):
            scheduler[idx] = CurrentThreadScheduler.singleton()
            gate[idx].release()

        for idx in (0, 1):
            threading.Thread(target=run, args=(idx,)).start()
            gate[idx].acquire()

        assert scheduler[0] is not None
        assert scheduler[1] is not None
        assert scheduler[0] is not scheduler[1]

    def test_currentthread_extend(self):
        class MyScheduler(CurrentThreadScheduler):
            pass

        scheduler = [
            MyScheduler(),
            MyScheduler.singleton(),
            MyScheduler.singleton(),
            CurrentThreadScheduler.singleton(),
        ]
        assert scheduler[0] is not scheduler[1]
        assert scheduler[1] is scheduler[2]
        assert scheduler[1] is not scheduler[3]

    def test_currentthread_now(self):
        scheduler = CurrentThreadScheduler()
        diff = scheduler.now - default_now()
        assert abs(diff) < timedelta(milliseconds=5)

    def test_currentthread_now_units(self):
        scheduler = CurrentThreadScheduler()
        diff = scheduler.now
        sleep(1.1)
        diff = scheduler.now - diff
        assert timedelta(milliseconds=1000) < diff < timedelta(milliseconds=1300)

    def test_currentthread_schedule(self):
        scheduler = CurrentThreadScheduler()
        ran = False

        def action(scheduler, state=None):
            nonlocal ran
            ran = True

        scheduler.schedule(action)
        assert ran is True

    def test_currentthread_schedule_block(self):
        scheduler = CurrentThreadScheduler()
        ran = False

        def action(scheduler, state=None):
            nonlocal ran
            ran = True

        t = scheduler.now
        scheduler.schedule_relative(0.2, action)
        t = scheduler.now - t
        assert ran is True
        assert t >= timedelta(seconds=0.2)

    def test_currentthread_schedule_error(self):
        scheduler = CurrentThreadScheduler()

        class MyException(Exception):
            pass

        def action(scheduler, state=None):
            raise MyException()

        with pytest.raises(MyException):
            scheduler.schedule(action)

    def test_currentthread_schedule_nested(self):
        scheduler = CurrentThreadScheduler()
        ran = False

        def action(scheduler, state=None):
            def inner_action(scheduler, state=None):
                nonlocal ran
                ran = True

            return scheduler.schedule(inner_action)

        scheduler.schedule(action)

        assert ran is True

    def test_currentthread_schedule_nested_order(self):
        scheduler = CurrentThreadScheduler()
        tests = []

        def outer(scheduler, state=None):
            def action1(scheduler, state=None):
                tests.append(1)

                def action2(scheduler, state=None):
                    tests.append(2)

                CurrentThreadScheduler().schedule(action2)

            CurrentThreadScheduler().schedule(action1)

            def action3(scheduler, state=None):
                tests.append(3)

            CurrentThreadScheduler().schedule(action3)

        scheduler.ensure_trampoline(outer)

        assert tests == [1, 2, 3]

    def test_currentthread_singleton_schedule_nested_order(self):
        scheduler = CurrentThreadScheduler.singleton()
        tests = []

        def outer(scheduler, state=None):
            def action1(scheduler, state=None):
                tests.append(1)

                def action2(scheduler, state=None):
                    tests.append(2)

                scheduler.schedule(action2)

            scheduler.schedule(action1)

            def action3(scheduler, state=None):
                tests.append(3)

            scheduler.schedule(action3)

        scheduler.ensure_trampoline(outer)

        assert tests == [1, 3, 2]

    def test_currentthread_ensuretrampoline(self):
        scheduler = CurrentThreadScheduler()
        ran1, ran2 = False, False

        def outer_action(scheduer, state=None):
            def action1(scheduler, state=None):
                nonlocal ran1
                ran1 = True

            scheduler.schedule(action1)

            def action2(scheduler, state=None):
                nonlocal ran2
                ran2 = True

            return scheduler.schedule(action2)

        scheduler.ensure_trampoline(outer_action)
        assert ran1 is True
        assert ran2 is True

    def test_currentthread_ensuretrampoline_nested(self):
        scheduler = CurrentThreadScheduler()
        ran1, ran2 = False, False

        def outer_action(scheduler, state):
            def inner_action1(scheduler, state):
                nonlocal ran1
                ran1 = True

            scheduler.schedule(inner_action1)

            def inner_action2(scheduler, state):
                nonlocal ran2
                ran2 = True

            return scheduler.schedule(inner_action2)

        scheduler.ensure_trampoline(outer_action)
        assert ran1 is True
        assert ran2 is True

    def test_currentthread_ensuretrampoline_and_cancel(self):
        scheduler = CurrentThreadScheduler()
        ran1, ran2 = False, False

        def outer_action(scheduler, state):
            def inner_action1(scheduler, state):
                nonlocal ran1
                ran1 = True

                def inner_action2(scheduler, state):
                    nonlocal ran2
                    ran2 = True

                d = scheduler.schedule(inner_action2)
                d.dispose()

            return scheduler.schedule(inner_action1)

        scheduler.ensure_trampoline(outer_action)
        assert ran1 is True
        assert ran2 is False

    def test_currentthread_ensuretrampoline_and_canceltimed(self):
        scheduler = CurrentThreadScheduler()
        ran1, ran2 = False, False

        def outer_action(scheduler, state):
            def inner_action1(scheduler, state):
                nonlocal ran1
                ran1 = True

                def inner_action2(scheduler, state):
                    nonlocal ran2
                    ran2 = True

                t = scheduler.now + timedelta(seconds=0.5)
                d = scheduler.schedule_absolute(t, inner_action2)
                d.dispose()

            return scheduler.schedule(inner_action1)

        scheduler.ensure_trampoline(outer_action)
        assert ran1 is True
        assert ran2 is False



import unittest

import reactivex
from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestRepeat(unittest.TestCase):
    def test_repeat_value_count_zero(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.repeat_value(42, 0)

        results = scheduler.start(create)

        assert results.messages == [on_completed(200)]

    def test_repeat_value_count_one(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.repeat_value(42, 1)

        results = scheduler.start(create)
        assert results.messages == [on_next(200, 42), on_completed(200)]

    def test_repeat_value_count_ten(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.repeat_value(42, 10)

        results = scheduler.start(create)
        assert results.messages == [
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_completed(200),
        ]

    def test_repeat_value_count_dispose(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.repeat_value(42, 10)

        results = scheduler.start(create, disposed=200)
        assert results.messages == []

    def test_repeat_value(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.repeat_value(42, -1)

        results = scheduler.start(create, disposed=201)
        assert results.messages[:6] == [
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
            on_next(200, 42),
        ]

    def test_repeat_observable_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(100, 1), on_next(200, 3), on_next(150, 2), on_completed(250)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat()))

        assert results.messages == [
            on_next(300, 1),
            on_next(350, 2),
            on_next(400, 3),
            on_next(550, 1),
            on_next(600, 2),
            on_next(650, 3),
            on_next(800, 1),
            on_next(850, 2),
            on_next(900, 3),
        ]
        assert xs.subscriptions == [
            subscribe(200, 450),
            subscribe(450, 700),
            subscribe(700, 950),
            subscribe(950, 1000),
        ]

    def test_repeat_observable_infinite(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(100, 1), on_next(150, 2), on_next(200, 3)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat()))

        assert results.messages == [on_next(300, 1), on_next(350, 2), on_next(400, 3)]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_repeat_observable_error(self):
        results = None
        scheduler = TestScheduler()
        ex = "ex"
        xs = scheduler.create_cold_observable(
            on_next(100, 1), on_next(150, 2), on_next(200, 3), on_error(250, ex)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat()))

        assert results.messages == [
            on_next(300, 1),
            on_next(350, 2),
            on_next(400, 3),
            on_error(450, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 450)]

    def test_repeat_observable_throws(self):
        scheduler1 = TestScheduler()
        xs = reactivex.return_value(11).pipe(ops.repeat())
        xs.subscribe(lambda x: _raise("ex"), scheduler=scheduler1)

        with self.assertRaises(RxException):
            scheduler1.start()

        scheduler2 = TestScheduler()
        ys = reactivex.throw("ex").pipe(ops.repeat())
        ys.subscribe(lambda ex: _raise("ex"), scheduler=scheduler2)

        with self.assertRaises(Exception):
            scheduler2.start()

        scheduler3 = TestScheduler()
        zs = reactivex.return_value(1).pipe(ops.repeat())
        d = zs.subscribe(on_completed=lambda: _raise("ex"), scheduler=scheduler3)

        scheduler3.schedule_absolute(210, lambda sc, st: d.dispose())
        scheduler3.start()

        # scheduler4 = TestScheduler()
        # xss = Observable.create(lambda o: _raise('ex')).repeat()
        # with self.assertRaises(RxException):
        #     xss.subscribe(scheduler=scheduler4)

    def test_repeat_observable_repeat_count_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(5, 1), on_next(10, 2), on_next(15, 3), on_completed(20)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat(3)))

        assert results.messages == [
            on_next(205, 1),
            on_next(210, 2),
            on_next(215, 3),
            on_next(225, 1),
            on_next(230, 2),
            on_next(235, 3),
            on_next(245, 1),
            on_next(250, 2),
            on_next(255, 3),
            on_completed(260),
        ]
        assert xs.subscriptions == [
            subscribe(200, 220),
            subscribe(220, 240),
            subscribe(240, 260),
        ]

    def test_repeat_observable_repeat_count_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(5, 1), on_next(10, 2), on_next(15, 3), on_completed(20)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat(3)), disposed=231)
        assert results.messages == [
            on_next(205, 1),
            on_next(210, 2),
            on_next(215, 3),
            on_next(225, 1),
            on_next(230, 2),
        ]
        assert xs.subscriptions == [subscribe(200, 220), subscribe(220, 231)]

    def test_repeat_observable_repeat_count_infinite(self):
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(100, 1), on_next(150, 2), on_next(200, 3)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat(3)))

        assert results.messages == [on_next(300, 1), on_next(350, 2), on_next(400, 3)]
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_repeat_observable_repeat_count_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_cold_observable(
            on_next(100, 1), on_next(150, 2), on_next(200, 3), on_error(250, ex)
        )
        results = scheduler.start(lambda: xs.pipe(ops.repeat(3)))

        assert results.messages == [
            on_next(300, 1),
            on_next(350, 2),
            on_next(400, 3),
            on_error(450, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 450)]

    def test_repeat_observable_repeat_count_throws(self):
        scheduler1 = TestScheduler()
        xs = reactivex.return_value(1).pipe(ops.repeat(3))
        xs.subscribe(lambda x: _raise("ex"), scheduler=scheduler1)

        with self.assertRaises(RxException):
            scheduler1.start()

        scheduler2 = TestScheduler()
        ys = reactivex.throw("ex1").pipe(ops.repeat(3))
        ys.subscribe(on_error=lambda ex: _raise("ex2"), scheduler=scheduler2)

        with self.assertRaises(RxException):
            scheduler2.start()

        # scheduler3 = TestScheduler()
        # zs = reactivex.return_value(1).repeat(100)
        # d = zs.subscribe(on_completed=lambda: _raise('ex3'), scheduler=scheduler3)

        # scheduler3.schedule_absolute(10, lambda sc, st: d.dispose())
        # scheduler3.start()

    #     xss = Observable.create(lambda o: _raise('ex4')).repeat(3)
    #     with self.assertRaises(RxException):
    #         xss.subscribe()



import unittest

import reactivex
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestTimeInterval(unittest.TestCase):
    def test_interval_timespan_basic(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.interval(100)

        results = scheduler.start(create)
        assert results.messages == [
            on_next(300, 0),
            on_next(400, 1),
            on_next(500, 2),
            on_next(600, 3),
            on_next(700, 4),
            on_next(800, 5),
            on_next(900, 6),
        ]

    # def test_interval_timespan_zero(self):
    #    scheduler = TestScheduler()

    #    def create():
    #        return reactivex.interval(0)

    #    results = scheduler.start(create, disposed=210)
    #    assert results.messages == [
    #            on_next(201, 0), on_next(202, 1), on_next(203, 2),
    #            on_next(204, 3), on_next(205, 4), on_next(206, 5),
    #            on_next(207, 6), on_next(208, 7), on_next(209, 8)]

    # def test_interval_timespan_negative(self):
    #    scheduler = TestScheduler()

    #    def create():
    #        return reactivex.interval(-1)

    #    results = scheduler.start(create, disposed=210)
    #    assert results.messages == [
    #            on_next(201, 0), on_next(202, 1), on_next(203, 2),
    #            on_next(204, 3), on_next(205, 4), on_next(206, 5),
    #            on_next(207, 6), on_next(208, 7), on_next(209, 8)]

    def test_interval_timespan_disposed(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.interval(1000)

        results = scheduler.start(create)
        assert results.messages == []

    def test_interval_timespan_observer_throws(self):
        scheduler = TestScheduler()
        xs = reactivex.interval(1)
        xs.subscribe(lambda x: _raise("ex"), scheduler=scheduler)

        with self.assertRaises(RxException):
            scheduler.start()



import unittest

import reactivex
from reactivex import operators as ops


class TestGroupByReduce(unittest.TestCase):
    def test_groupby_count(self):
        res = []
        counts = reactivex.from_(range(10)).pipe(
            ops.group_by(lambda i: "even" if i % 2 == 0 else "odd"),
            ops.flat_map(
                lambda i: i.pipe(
                    ops.count(),
                    ops.map(lambda ii: (i.key, ii)),
                )
            ),
        )

        counts.subscribe(on_next=res.append)
        assert res == [("even", 5), ("odd", 5)]

    def test_window_sum(self):
        res = []
        reactivex.from_(range(6)).pipe(
            ops.window_with_count(count=3, skip=1),
            ops.flat_map(
                lambda i: i.pipe(
                    ops.sum(),
                )
            ),
        ).subscribe(on_next=res.append)

        assert res == [3, 6, 9, 12, 9, 5, 0]
